@@ -21,11 +21,11 @@ gamlssMX  <- function ( formula = formula(data),
                       ...)
  {
 #----------------------------------------------------------
-# extra functions within
-# for getting the comulative function
+# extra functions within 
+# for getting the commulative function
 get.the.p.function <- function(object, ...)
 {
-gamlss.bi.list <- c("BI", "Binomial", "BB", "Beta Binomial")
+#gamlss.bi.list <- c("BI", "Binomial", "BB", "Beta Binomial")
 if (!is.gamlss(object))  stop(paste("This is not an gamlss object", "\n", ""))
    fname <- object$family[[1]]
  DistPar <- object$parameters
@@ -33,9 +33,9 @@ if (!is.gamlss(object))  stop(paste("This is not an gamlss object", "\n", ""))
     dfun <- paste("p",fname,sep="")
  # binomial denominators
  switch(nopar,  
-    {pfun <- if (fname%in%gamlss.bi.list)  eval(call(dfun,q=object$y, bd=object$bd,  mu=fitted(object,"mu")))
+    {pfun <- if (fname%in%gamlss:::.gamlss.bi.list)  eval(call(dfun,q=object$y, bd=object$bd,  mu=fitted(object,"mu")))
              else  eval(call(dfun,q=object$y, mu=fitted(object,"mu")))},
-    {pfun <- if (fname%in%gamlss.bi.list)  eval(call(dfun,q=object$y, bd=object$bd,  mu=fitted(object,"mu"),sigma=fitted(object,"sigma")))
+    {pfun <- if (fname%in%gamlss:::.gamlss.bi.list)  eval(call(dfun,q=object$y, bd=object$bd,  mu=fitted(object,"mu"),sigma=fitted(object,"sigma")))
              else  eval(call(dfun,q=object$y, mu=fitted(object,"mu"), sigma=fitted(object,"sigma"))) },
     {pfun <- eval(call(dfun,q=object$y, mu=fitted(object,"mu"), sigma=fitted(object,"sigma"), nu=fitted(object,"nu")))},
     {pfun <- eval(call(dfun,q=object$y, mu=fitted(object,"mu"), sigma=fitted(object,"sigma"), nu=fitted(object,"nu"), tau=fitted(object,"tau")))})
@@ -45,16 +45,16 @@ pfun
 #=---------------------------------------------------------
 get.likelihood <- function(obj)
 {
-gamlss.bi.list <- c("BI", "Binomial", "BB", "Beta Binomial")
+#gamlss.bi.list <- c("BI", "Binomial", "BB", "Beta Binomial")
 if (!is.gamlss(obj))  stop(paste("This is not an gamlss object", "\n", ""))
    fname <- obj$family[[1]]
  DistPar <- obj$parameters
    nopar <- length(DistPar) 
     dfun <- paste("d",fname,sep="")
  switch(nopar,  
-    {lik <-if (fname%in%gamlss.bi.list)  eval(call(dfun,y=obj$y, bd=obj$bd,  mu=fitted(obj,"mu")))
+    {lik <-if (fname%in%gamlss:::.gamlss.bi.list)  eval(call(dfun,y=obj$y, bd=obj$bd,  mu=fitted(obj,"mu")))
              else   eval(call(dfun, y=obj$y, mu=fitted(obj)))},
-    {lik <-if (fname%in%gamlss.bi.list)  eval(call(dfun,y=obj$y, bd=obj$bd,  mu=fitted(obj,"mu"), sigma=fitted(obj,"sigma") ))
+    {lik <-if (fname%in%gamlss:::.gamlss.bi.list)  eval(call(dfun,y=obj$y, bd=obj$bd,  mu=fitted(obj,"mu"), sigma=fitted(obj,"sigma") ))
              else   eval(call(dfun,y=obj$y, mu=fitted(obj), sigma=fitted(obj,"sigma"))) },
     {lik <- eval(call(dfun,y=obj$y, mu=fitted(obj), sigma=fitted(obj,"sigma"), nu=fitted(obj,"nu")))},
     {lik <- eval(call(dfun,y=obj$y, mu=fitted(obj), sigma=fitted(obj,"sigma"), nu=fitted(obj,"nu"), tau=fitted(obj,"tau")))})
@@ -93,52 +93,50 @@ library(nnet)
 ## checking for NA in the data 
  if(!missing(data) & any(is.na(data)))   
       stop("The data contains NA's, use data = na.omit(mydata)") 
-
-
 ## prob are the starting values of the pi's   
  if(zero.component == TRUE)
  { KK <- K+1 
- if (is.null(prob)) 
+  if (is.null(prob)) 
    { 
      prob <- rep(1/KK, KK)
    }
- else
+  else
    {
     if (length(prob)!=KK) stop("the length of prob must be  K+1(for zero component) ")
    } 
  }
  else
  {
- if (is.null(prob)) 
-   { 
+   if (is.null(prob)) 
+     { 
      prob <- rep(1/K, K)
-   }
+     }
  else
    {
     if (length(prob)!=K) stop("the length of prob must be K")
    } 
  }
 ## expand the formula if not list
-  if (!is.list(formula))
-   {
+ if (!is.list(formula))
+  {
    allFormula <- vector("list",K)
    for (i in 1:K) allFormula[[i]] <- formula
-   }
-   else allFormula <- formula
+  }
+ else allFormula <- formula
 ## expand the family if not list
-  if (!is.list(family))
-   {
+ if (!is.list(family))
+  {
    allFamily <- vector("list",K)
    for (i in 1:K) allFamily[[i]] <- family
-   }
-   else allFamily <- family
+  }
+  else allFamily <- family
 ## check that the length for both the family and formula is K
 if (length(allFamily)!=K) stop("the length of the list for the family is not equal the number of componets in the mixture")
 if (length(allFormula)!=K) stop("the length of the list for the formula is not equal the number of componets in the mixture")
 ## get response for his length
  Y <- model.extract(model.frame(allFormula[[1]],data=data),"response")
  N <-  if(is.null(dim(Y))) length(Y) else dim(Y)[1]   # calculate the dimension for y  
-pweights. <<- if (missing(weights)) rep(1,N) else weights  
+pweights. <- pweights. <<- if (missing(weights)) rep(1,N) else weights  
 ## get things from control
   set.seed(control$seed) # set the seed 
   prob.sample <- if (is.null(control$sample)) 10/N else control$sample  
@@ -148,25 +146,26 @@ pweights. <<- if (missing(weights)) rep(1,N) else weights
 ## get starting values for the weights (posterior probabilities)
  for (i in 1:K)
    {          wSam <- sample(c(0,1), N, replace = TRUE, prob=c(1-min(.5,prob.sample),min(.5,prob.sample)))  
-              ww. <<- wSam # W[,i] # put in 
+              ww. <- ww. <<- wSam # W[,i] # put in 
              #data1 <<- data.frame(data, ww, pweights)
     allModels[[i]] <- gamlss(allFormula[[i]], weights=ww.*pweights., data = data, family = allFamily[[i]], control=g.control, ...)
-             W[,i] <- get.likelihood(allModels[[i]]) } 
-    if(zero.component == TRUE) W[,KK] <- prob[KK]*ifelse(Y==0,1,0)
-            SumLik <-rowSums(W)
-                 W <- W/SumLik
-                 W <- ifelse(W<1e-20,0,W)
-              prob <- colSums(W)/N
+             W[,i] <- get.likelihood(allModels[[i]]) 
+   } 
+ if(zero.component == TRUE) W[,KK] <- prob[KK]*ifelse(Y==0,1,0)
+      SumLik <- rowSums(W)
+           W <- W/SumLik
+           W <- ifelse(W<1e-20,0,W)
+        prob <- colSums(W)/N
  # -2 * logLikelihood
-       newdv <-  -2*sum(log(SumLik))    # the global deviance
+       newdv <- -2*sum(log(SumLik))    # the global deviance
        olddv <- newdv + 1  
     iter.num <- 1 
  trace.print <- 0 
     dev.fits <- rep(0,control$n.cyc)
 #------------------------------------------------
 # if require model for prior propabilities pi's
-modelPi <- if (pi.formula[[2]]==1) FALSE else TRUE
-if (modelPi)
+     modelPi <- if (pi.formula[[2]]==1) FALSE else TRUE
+  if (modelPi)
    {
     if (missing(data)) stop("the data argument is needed if pi's are modelled")
      if(zero.component == TRUE) 
@@ -180,14 +179,14 @@ if (modelPi)
       fac.fit <- gl(K,N) # get the factor
       }
      res.var <- model.matrix(~fac.fit-1) # get the matrix of zeroes ans ones
-   form.prob <- formula(paste("res.var~", pi.formula[[2]])) # get the formula
+   form.prob <-  update(pi.formula, res.var~.)# formula(paste("res.var~", pi.formula[[2]])) # get the formula 
         PROB <-  matrix(prob, ncol=K, nrow=N, byrow=TRUE) # expand prior prob
    }
 ## EM starts here--------------------------------- 
  while ( abs(olddv-newdv) > control$cc && iter.num < control$n.cyc ) # MS Wednesday, June 26, 2002 
   {
   for (i in 1:K)
-    {          ww. <<- W[,i]
+    {          ww. <- ww. <<- W[,i]
     allModels[[i]] <- gamlss(allFormula[[i]], weights=ww.*pweights., data = data, family = allFamily[[i]], control=g.control,...)
              W[,i] <- if (modelPi) PROB[,i]*get.likelihood(allModels[[i]]) else prob[i]*get.likelihood(allModels[[i]])
     } 
@@ -203,7 +202,8 @@ if (modelPi)
              newdv <- sum(di)
        if (modelPi)
           {
-          dataKK.$wWw <- as.vector(W)
+          dataKK.$wWw <- wWw <- as.vector(W)
+          dataKK.$res.var <- res.var
           mult.mod <- multinom(form.prob, weights=wWw, data=dataKK., trace=FALSE)
            fitted(mult.mod)[1:11,]
               PROB <- fitted(mult.mod)[1:N,] # take only N of them 
@@ -383,9 +383,9 @@ terms.gamlssMX <- function(x, K=1, ...)
 terms(x$models[[K]], ...)
 }
 #----------------------------------------------------------------------------------------
-predict.gamlssMX <- function(x, K=1, ...)
+predict.gamlssMX <- function(object, K=1, ...)
 {
-predict(x$models[[K]], ...)
+predict(object$models[[K]], ...)
 }
 #----------------------------------------------------------------------------------------
 residuals.gamlssMX <- function(object,...)
@@ -429,6 +429,46 @@ gamlssMXfitscall[[1]] <- as.name("gamlssMX")
  model
 }
 #----------------------------------------------------------------------------------------
+update.gamlssMX <- function (object, 
+                          formula., 
+                          ..., 
+                          what = c("mu", "sigma", "nu", "tau"),
+                          evaluate = TRUE) 
+{
+    call <- object$call
+    if (is.null(call)) 
+        stop("need an object with call component")
+    extras <- match.call(expand.dots = FALSE)$...
+    if (!missing(formula.)) 
+        {
+        what <- match.arg(what) 
+        if (what=="mu") 
+         { call$formula <- update.formula(formula(object,what), formula.) }
+        else  
+         {
+          call[[paste(what,"formula",sep=".")]] <- 
+         if (length(update.formula(formula(object,what), formula.))==2)
+          update.formula(formula(object,what), formula.)
+         else
+          update.formula(formula(object,what), formula.)[-2]
+         }
+        }
+    if (length(extras) > 0) 
+        {
+        existing <- !is.na(match(names(extras), names(call)))
+        for (a in names(extras)[existing]) call[[a]] <- extras[[a]]
+        if (any(!existing)) 
+           {
+            call <- c(as.list(call), extras[!existing])
+            call <- as.call(call)
+           }
+        }
+    if (evaluate) 
+        eval(call, parent.frame())
+    else call
+}
+######################################################################################
+#
 #----------------------------------------------------------------------------------------
 #----------------------------------------------------------------------------------------
 # this is a function to calculate the pdf of y fY(y)=p1*f1(y)+p2*f(y), of a MX model
@@ -444,27 +484,45 @@ dMX <- function(y,
                  log = FALSE,
                  ...)
   {
-  gamlss.bi.list <- c("BI", "Binomial", "BB", "Beta Binomial")
+#  gamlss.bi.list <- c("BI", "Binomial", "BB", "Beta Binomial")
+#  .gamlss.bi.list<-c("BI", "Binomial", "BB", "Beta Binomial")
  ## checking the probabilities 
      sump <- 0
-        K <- length(pi) 
+        K <- length(pi) ## set the length using the length of propabilities
   for (i in 1:K) sump <-pi[[i]]+ sump 
   if (any(sump!=1)) stop(paste("the vector pi should sum to 1"))
  ## get the length of y and the length of the parameters
  if(is.null(dim(y))) N <- length(y) else N <- dim(y)[1]  
-     Prob <- matrix(1, nrow=N, ncol=K ) # rep(prob,rep(N,K))  
-     for (i in 1:K)
+       Prob <- matrix(1, nrow=N, ncol=K ) # rep(prob,rep(N,K)) 
+     for (i in 1:K) # go throught the components
    { 
         fam <-  as.gamlss.family(family[[i]])
       fname <- fam$family[[1]]
     DistPar <- fam$parameters
       nopar <- length(DistPar)  
        dfun <- paste("d",fname,sep="") 
+       if (fname%in%gamlss:::.gamlss.bi.list) # for binomial data
+    {
+     if (NCOL(y) == 1)
+        {
+        y <- y 
+       bd <- rep(1, length(y))
+        }
+     else if (NCOL(y) == 2)
+        {
+        y <- y[,1]
+       bd <- y[,1]+y[,2]
+        }
+     else
+        {
+      stop("wrong response variable for binomial data")
+        }
+     }           
    switch(nopar,  
-      {lik <- if (fname%in%gamlss.bi.list) eval(call(dfun, y=y, bd=bd[[i]],  mu=mu[[i]]))
+      {lik <- if (fname%in%gamlss:::.gamlss.bi.list){eval(call(dfun, y=y, bd=bd,  mu=mu[[i]]))}
             else   eval(call(dfun, y=y, mu=mu[[i]]))
       },
-      {lik <- if (fname%in%gamlss.bi.list) eval(call(dfun, y=y, bd=bd[[i]],  mu=mu[[i]], sigma=sigma[[i]]))
+      {lik <- if (fname%in%gamlss:::.gamlss.bi.list) eval(call(dfun, y=y, bd=bd,  mu=mu[[i]], sigma=sigma[[i]]))
             else   eval(call(dfun, y=y, mu=mu[[i]], sigma=sigma[[i]])) 
       },
       {lik <- eval(call(dfun,y=y, mu=mu[[i]], sigma=sigma[[i]], nu=nu[[i]]))},
@@ -503,7 +561,24 @@ pMX <- function(q,
       fname <- fam$family[[1]]
     DistPar <- fam$parameters
       nopar <- length(DistPar)  
-       dfun <- paste("p",fname,sep="") 
+       dfun <- paste("p",fname,sep="")
+        if (fname%in%gamlss:::.gamlss.bi.list) # for binomial data
+    {
+     if (NCOL(q) == 1)
+        {
+        q <- q 
+       bd <- rep(1, length(q))
+        }
+     else if (NCOL(q) == 2)
+        {
+        q <- q[,1]
+       bd <- q[,1]+q[,2]
+        }
+     else
+        {
+      stop("wrong response variable for binomial data")
+        }
+     }            
    switch(nopar,  
      {lik <- if (fname%in%gamlss.bi.list) eval(call(dfun, q=q, bd=bd[[i]],  mu=mu[[i]]))
             else   eval(call(dfun, q=q, mu=mu[[i]]))
@@ -520,43 +595,3 @@ pMX <- function(q,
        fy 
   }
 #----------------------------------------------------------------------------------------
-update.gamlssMX <- function (object, 
-                          formula., 
-                          ..., 
-                          what = c("mu", "sigma", "nu", "tau"),
-                          evaluate = TRUE) 
-{
-    call <- object$call
-    if (is.null(call)) 
-        stop("need an object with call component")
-    extras <- match.call(expand.dots = FALSE)$...
-    browser()
-    if (!missing(formula.)) 
-        {
-        what <- match.arg(what) 
-        if (what=="mu") 
-         { call$formula <- update.formula(formula(object,what), formula.) }
-        else  
-         {
-          call[[paste(what,"formula",sep=".")]] <- 
-         if (length(update.formula(formula(object,what), formula.))==2)
-          update.formula(formula(object,what), formula.)
-         else
-          update.formula(formula(object,what), formula.)[-2]
-         }
-        }
-    if (length(extras) > 0) 
-        {
-        existing <- !is.na(match(names(extras), names(call)))
-        for (a in names(extras)[existing]) call[[a]] <- extras[[a]]
-        if (any(!existing)) 
-           {
-            call <- c(as.list(call), extras[!existing])
-            call <- as.call(call)
-           }
-        }
-    if (evaluate) 
-        eval(call, parent.frame())
-    else call
-}
-######################################################################################
